/*
* Copyright (c) 2007-2017 TeamSpeak Systems
*/

#include "common.h"

#include <teamspeak/public_errors.h>

#if defined(_MSC_VER) && !defined(_CRT_SECURE_NO_WARNINGS)
#define _CRT_SECURE_NO_WARNINGS
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifdef _WIN32
#define WIN32_LEAN_AND_MEAN
#include <Windows.h>
#else
#include <time.h>
#endif

// for calculating the circle
#define _USE_MATH_DEFINES
#include <math.h>
#include <assert.h>

#define CHECK_ERROR_FATAL(x) check_error_ok(x, __FILE__ ,__LINE__, true)
#define CHECK_ERROR(x) check_error_ok(x, __FILE__ ,__LINE__, false)

bool check_error_ok(unsigned int error, const char *file, unsigned int line, bool do_exit)
{
    if (error != ERROR_ok && error != ERROR_ok_no_update)
    {
        printf("Error (%d) occurred on line %d in %s\n", error, line, file);
        if (do_exit)
        {
            printf("Press Enter to terminate\n");
            getchar();
            exit(1);
        }

        return false;
    }
    return true;
}

/*A function so suspend the current thread for a specified amount of milliseconds*/
void sleep_ms(int milliseconds)
{
#ifdef _WIN32
    Sleep(milliseconds);
#else
    struct timespec ts;
    ts.tv_sec = milliseconds / 1000;
    ts.tv_nsec = (milliseconds % 1000) * 1000000;
    nanosleep(&ts, NULL);
#endif
}

/*Return program path ending in path separator, or an empty string if there is
none*/
char* program_path(const char* program_invocation)
{
    char* end;
#ifdef _WIN32
    char pathsep = '\\';
#else
    char pathsep = '/';
#endif
    static char result[4096]; /*note that this entire array is zeroed*/

    if (program_invocation == NULL)
        program_invocation = "";

    end = strrchr(program_invocation, pathsep);
    if (end)
    {
        const int count = (int)(end + 1 - program_invocation);
        strncpy(result, program_invocation, count);
        result[count] = '\0';
    }
    else
        result[0] = '\0';

    return result;
}

#define TS_LOOKUP_TABLE_ENTRIES_3D 20
const int k3dLookupTableEntries = TS_LOOKUP_TABLE_ENTRIES_3D;
float lookup[TS_LOOKUP_TABLE_ENTRIES_3D * 2]; // we have x and y in our array;
#undef TS_LOOKUP_TABLE_ENTRIES_3D

void create_circle_lookup_table(unsigned int radius)
{
    const double kTwoPi = 2 * M_PI;
    const double kStep = kTwoPi / k3dLookupTableEntries;
    double theta = 0;
    for (int i = 0; i < k3dLookupTableEntries; ++i)
    {
        assert(theta < kTwoPi);
        const float x = (float)(radius * cos(theta));
        const float y = (float)(radius * sin(theta));
        const int lookup_position = i * 2; // we have x and y in our array;
        lookup[lookup_position] = x;
        lookup[lookup_position + 1] = y; 
        theta += kStep;
    }
}

void get_random_position_on_circle(TS3_VECTOR* ts_vector)
{
    if (ts_vector == NULL)
        return;

    const int random_lookup_entry = rand() % k3dLookupTableEntries;
    const int lookup_position = random_lookup_entry * 2; // we have x and y in our array
    ts_vector->x = lookup[lookup_position];
    ts_vector->y = lookup[lookup_position + 1];
}

int get_lookup_table_entry_count()
{
    return k3dLookupTableEntries;
}

void get_position_on_circle_entry(TS3_VECTOR* ts_vector, int lookup_entry)
{
    if (ts_vector == NULL)
        return;

    lookup_entry++;
    if (lookup_entry >= k3dLookupTableEntries)
        lookup_entry = 0;

    const int lookup_position = lookup_entry * 2;
    ts_vector->x = lookup[lookup_position];
    ts_vector->y = lookup[lookup_position + 1];
}
